#!/bin/sh
# \
exec fltkwish "$0" -- ${1+"$@"}
#
# --- extract.tcl --- Get weather observations and display them
#
# Copyright(C) I.B.Findleton, 2002. All Rights Reserved.
#
package require TclUtils
package require httpcopy

set Maps(Verbose) 0

set Data(Station) YUL
set Data(BaseURL) www.weatheroffice.ec.gc.ca
set Data(Script) scripts/citygen.pl?client=ECCDN_e&city=

set Data(Names) [list "Station" "ValidTime" "Temperature" "Pressure" "Visibility" "Humidity" \
			"DewPoint" "Wind" "WindSpeed" "WindGusts" "WindChill" "Humidex" \
			"CurrentWeather" "Icon" "MaxTemp" "MinTemp" "MeanTemp" "Precip" \
			"Sunrise" "Sunset" "Moonrise" "Moonset" ]

set Data(Update) 1

# Make a request for a city

proc MakeQuery { name } {

	global Data
	
	return "$Data(BaseURL)/$Data(Script)$name"
	}

# Get the station data from the web site

proc LoadStationData { name } {

	global Data
	
	set Data(FileName) $name.txt
	
	if { [catch { ::http::copy [MakeQuery $name] $Data(FileName) } reason] } {
		puts stderr $reason
		return 0
		}
		
	return 1
	}

# Get the observation data

proc GetObservedData { name } {

	set fd [open $name r]
	if { $fd == "" } {
		puts stderr "Failed to open file $name!"
		return ""
		}

	set txt [read $fd]

	close $fd
	
	return [CleanHTMLBlock $txt]
	}

# Strip the HTML tags from a block of data

proc CleanHTMLBlock { txt } {

	regsub -all {<IMG|<img} $txt {IMG} txt
	regsub -all {<([^>])+>} $txt {} txt
	regsub -all {^([[:space:]])*\n} $txt {} txt
	regsub -all {  } $txt {} txt
	regsub -all {\n^} $txt {} txt
	
	puts $txt
	return $txt
	}

# Parse the observation data

proc ParseData { txt } {

	global Data
	
	set list [split $txt \n]
	set idx 0
	foreach line $list {
		if { [string length $line] != 0 } {
			set parts [split $line :]
			switch -exact -- [string trim [lindex $parts 0]] {
		
"Observed on"		{	set Data(ValidTime) [lindex $parts 1][lindex $parts 2]
							}
"Temp."				{	regsub {&deg;+} [lindex $parts 1] { } Data(Temperature)
							set Data(Temperature) "$Data(Temperature) [lindex $parts 2]"
							}
"Pressure"			{	set Data(Pressure) [lindex $parts 1]
							}
"Visibility"		{	set Data(Visibility) [lindex $parts 1]
							}
"DewPoint"			{	regsub {&deg;+} [lindex $parts 1] { } Data(DewPoint)
							set Data(DewPoint) "$Data(DewPoint) [lindex $parts 2]"
							}
"Humidity"			{	set Data(Humidity) [lindex $parts 1]
							}
"Wind"				{	set Data(Wind) [lindex $parts 1]
							if { [string first gust $Data(Wind)] != -1 } {
								regexp -all {([-0123456789.%])+} $Data(Wind) Data(WindSpeed) Data(WindGusts)
							} else {
								regexp -all {([-0123456789.%])+} $Data(Wind) Data(WindSpeed)
								}
							}
"WindChill"			{	set Data(WindChill) [lindex $parts 1]
							}
"Humidex"				{	set Data(WindChill) [lindex $parts 1]
						}
"Max Temp."			{	regsub {&deg;+} [lindex $list [expr $idx + 1]] { } Data(MaxTemp)
						}
"Min Temp."			{	regsub {&deg;+} [lindex $list [expr $idx + 1]] { } Data(MinTemp)
						}
"Mean Temp."			{	regsub {&deg;+} [lindex $list [expr $idx + 1]] { } Data(MeanTemp)
						}
"Sunrise" 			{	set Data(Sunrise) [lindex $list [expr $idx + 1]]
						}
"Sunset"				{	set Data(Sunset) [lindex $list [expr $idx + 1]]
						}
"Moonrise"			{	set Data(Moonrise) [lindex $list [expr $idx + 1]]
						}
"Moonset"			{	set Data(Moonset) [lindex $list [expr $idx + 1]]
						}
"Precip. Total"		{	set Data(Precip) [lindex $list [expr $idx + 1]]
						}

default				{
							}
						} ;#End of switch cases
				}
			incr idx	
			}
	set idx [lsearch -glob $list "*Observed on*"]
	if { $idx != -1 } {
		set Data(Icon) [lindex [split [lindex $list [incr idx]] ] 1]
		regsub -all {src=+|\"+} $Data(Icon) {} Data(Icon)
		set Data(CurrentWeather) [lindex $list [incr idx]]
	} else {
		set Data(CurrentWeather) "???"
		set Data(Icon) "???"
		}

	}
	
# Update the station data

proc UpdateStation { } {

	global Data
	
	set Data(Update) 0

	foreach name $Data(Names) {
		if { $name != "Station" } {
			set Data($name) Missing
			}
		}
			
	if { [LoadStationData $Data(Station)] } {
		ParseData [GetObservedData	$Data(FileName)]
		}
		
	set Data(Update) 1
	}
	
# Parse the parameter list

foreach { option value } $argv {
	switch -exact -- [string tolower $option] {
	
"-s"			-
"-station"	{	set Data(Station) [string toupper $value] }

"-v"			-
"-verbose"	{	set Maps(Verbose) [IsEqual [string tolower $value] "yes" 1 0]
				}
				
default		{	puts "Option $option is not supported!"
				Exit
				}
			} ;# End of switch cases
		}	;# End of foreach loop			
	
# Create an interface to display the data

catch { Destroy t }

set id 0

set f [Package t.p -orientation vertical]
foreach name $Data(Names) {
	if { $name == "Station" } {
		LabeledInput $f.lt[incr id] -label $name -variable Data($name) -ratio 0.6 \
			-width 300 -command UpdateStation
	} else { 
		LabeledText $f.lt[incr id] -label $name -variable Data($name) -ratio 0.6 -width 300
		}
	}
	
set f1 [Package $f.actions -relief ridge -orientation horizontal -width 250]

Option add Button.width 125

Button $f1.update -text Update -command UpdateStation -tooltip "Update station data" \
			-statevariable Data(Update)
Button $f1.quit -text Quit -command Exit -tooltip "End this application" \
			-statevariable Data(Update)
	
Show t

Wm title t "Station Monitor V1.0"

UpdateStation
