#!/bin/sh
# \
exec tclsh "$0" -- ${1+"$@"}
#
# --- stnplot.tcl --- Station plotting application
#
# Copyright(C) I.B.Findleton, 2002. All Rights Reserved.
#
# This script monitors the observed weather data available on the web site
# of the Canadian Weather Service's main public site. The data for the
# selected list of stations is plotted as a standard station plot or is
# displayed as a weather depiction image. The data for the station is
# automatically updated by polling of the web site.
#
# The format of the command line call to this script is:
#
#	stnplot.tcl -s stn,stn,... -v verbose -m mode -w widgets -?
#				-h history -g gmt -d database -u units
#
# Where the stn values are station identifiers and the verbose parameter is
# either yes or no. The mode value can be plot or depiction. Any number of 
# station identifiers can be provided. The script will wrap the station 
# plots into a scrolled window if there are more than 4 stations specified.
#
# The mode specifies the default display mode. If it is plot, then something
# like the standard station plot model is used. If it is not plot, then an
# image depicting current conditions will be used.
#
# The widgets value is the number of widgets to show in the display window
# without having to scroll the list. Any number of stations can be displayed,
# but only this number will be visible at any one time. This is used to 
# conserve window real estate on your desktop.  
#
# Notes:
#	1) Canada is a metric country, so all data is metric unless you
#	   specify the -units imperial option
#	2) By default, the time values are GMT. Use -g 0 for local time
#	3) You need to use the station aviation identifiers (i.e YUL, KMIA)
#	   for the stations. Canadian stations need 3 letters, US stations
#	   have 4 letters, the first of which is K. You can also use the C
#	   for the Canadian stations, if you are a strict purist!
#	4) If you are on Linux/Unix and have the Tclgdbm extension, the data
#	   you download will be saved in a convenient database. On windows,
#	   you will need to roll your own archive by modifying this script.
#

# Check for a package needed by this script

proc CheckForPackage { name { version  "" } } {

# All packages needed for this script are available here...

	global tcl_platform
	
	set site "http://pages.infinit.net/cclients/software.htm"
	
# The following code assumes that UNIX users have netscape, Windows users have something.
# On other systems you have to roll your own code....

	if { [string length $version] != 0 } {
		set result [catch { package require $name $version } reason]
	} else {
		set result [catch { package require $name } reason]
		}
		
	if { $result } {
		puts stderr "Did not get package $name $version : $reason"
		puts stderr "This script requires the $name $version package!"
		puts stderr "Get it at $site"
		if { $tcl_platform(platform) == "unix" } {
			catch { exec netscape $site & } 
		} else {
			catch { exec start $site & }
			}
		exit
		}
		
	return $reason
	}
	
# These are small utility packages that you can get from my web site

CheckForPackage TclUtils
CheckForPackage httpcopy

# If you are on UNIX and have the Tclgdbm package you get a database of history data

if { [catch { package require Tclgdbm } reason] } {
	set Data(HasGdbm) 0
} else {
	set Data(HasGdbm) 1
	}

# This statement turns of http related download messages.

set Maps(Verbose) 0

# Set some default values. These stations are around Montreal, Canada, and
# Miami, just to make winter dreams more real....

set Data(Station) YUL,YMX,YHU,YOW,YSC,YYZ,YQB,KMIA

# This is the defaule site for the web server. It is an alias for several
# servers of different types.

set Data(BaseURL) weatheroffice.ec.gc.ca
set Data(Script) scripts/citygen.pl
set Data(Client,Canada) ?client=ECCDN_e&city=
set Data(Client,USA) ?client=ECUSA_e&city=
set Data(Token) {}

# We need at most 4 hours for the pressure tendency calculation

set Data(HistoryLimit) 4

if { $tcl_platform(platform) == "unix" } {
	set host [exec hostname]
} else {
	set host windows
	}

# This code configures the script for a few of my machines. 
# You probably want to change this for your own situation. It sets
# up the location of the data storage area on my computers.

switch -exact -- $host {

ganymede		{
				set Data(DefaultRoot) /public/Database/weather.data
				}
invicta		{
				set Data(DefaultRoot) /database/weather.data
				}
windows		{
				set Data(DefaultRoot) /
				}
				
default		{	set Data(DefaultRoot) .
				}
			} ;# End of switch cases

# The log file collects information about what is happening. You may not need it.

set Data(LogFile) /var/log/stations

# This is where image icons are stored. Once you have them all, the
# script will not download things from the web site. This makes things
# faster and less resource consuming, I presume.

set Data(Images) $Fltk(Library)/scripts/images/weather

set Data(DefaultUnits) ""

if { $Data(HasGdbm) } {
	set Data(Database) $Data(DefaultRoot)/station.observations.gdbm
} else {
	}

set Data(MissingValue) "???"	;# Indicates a field is not present

set Data(GMT) 1			;# Times in GMT

# Translate the current sky cover to a name for a background image

set SkyConditions {

	{	"Mostly Cloudy"	"Cloudy"					6	}
	{	"Sunny"			"Sunny"					0	}
	{	"Clear"			"Clear"					0	}
	{	"Overcast"		"Cloudy"					8	}
	{  	"Mainly Clear"		"Cloudy_with_clear_periods"	2	}
	{  	"Mainly Sunny"		"Sunny_with_cloudy_periods"	2	}
	{	"Cloudy"			"Cloudy" 					4	}

	}

# Find sky condition

proc FindSkyCondition { station str hour } {

	global SkyConditions Data

	foreach entry $SkyConditions {
		if { [string first "[lindex $entry 0]" $str] != -1 } {
#			set Data($station,Image,$hour) $Data(Images)/[lindex $entry 1].bmp
			return [lindex $entry 2]
			}
		}

	return "-1"
	}

# Translate the obstructions to vision. Order is important in this list!
# It appears that this site uses the simple term, like "rain", to mean "light rain"
# and qualifies more intense events with the "moderate" and "heavy" modifiers.

set Obstructions {

	{	"light rainshower"	"RW-" 	"Showers"	}
	{	"moderate rainshower"	"RW"	"Rainy"	}
	{	"heavy rainshower"	"RW+"	"Rainy"	}
	{	"rainshower"		"RW-"	"Showers"	}
	{	"light snowshower"	"SW-"	"Cloudy_with_snow"	}
	{	"moderate snowshower"	"SW"	"Cloudy_with_snow"	}
	{	"heavy snowshower"	"SW+"	"Snowing"	}
	{	"snowshower"		"SW-"	"Cloudy_with_snow"	}
	{	"snow grains"		"SG"		"Cloudy_with_snow"	}
	{	"moderate freezing rain" "ZR" "Freezing_rain"	}
	{	"heavy freezing rain"	"ZR+" "Freezing_rain"	}
	{	"freezing rain"		"ZR-" "Freezing_rain"	}
	{	"light rain"		"R-"		"Rainy"	}
	{	"moderate rain"		"R"	"Rainy"	}
	{	"heavy rain"		"R+"		"Rainy"	}
	{	"rain"			"R-"		"Rainy"	}
	{	"precipitation"		"R-"	"Rainy"	}	
	{	"light snow"		"S-"		"Snowing"		}
	{	"moderate snow"		"S"	"Snowing"	}
	{	"heavy snow"		"S+"		"Snowing"		}
	{	"drifting snow"		"DS"	"Blowing_snow" }
	{	"blowing snow"		"BS"		"Blowing_snow" }
	{	"snow"			"S-"		"Snowing"	}
	{	"light drizzle"		"L-"	"Drizzle"	}
	{	"moderate drizzle"	"L"		"Drizzle"	}
	{	"freezing drizzle"	"ZL-"	"Freezing_rain"	}
	{	"drizzle"			"L-"		"drizzle"	}
	{	"heavy thunderstorm" 	"TRW+" "Thundershowers"	}
	{	"thunderstorm"		"TRW"	"Thundershowers"	}
	{	"thunder"			"T"		"Thundershowers"	}
	{	"haze"			"H"		"Haze"	}
	{	"smoke"			"K"		"Smoke"	}
	{	"mist"			"M"		"Mist"	}
	{	"fog"				"F"	"Foggy"	}
	{	"dust"			"D"		"Dust" }

	}

# Find all obstructions mentioned in a string

proc FindObstructions { station str hour } {

	global Obstructions Data

	set result ""
	
	set list [split $str ,]

	foreach item $list {
		foreach entry $Obstructions {
			if { [regexp -nocase "[lindex $entry 0]" $item] != 0 } {
				append result [lindex $entry 1]
				break
				}
			}
		}
	return $result
	}

# This is the list of things that can be shown by the station model

set Data(Names) [list "Temperature" "Pressure" "Visibility" "Humidity" \
			"DewPoint" "WindDirection" "WindSpeed" "WindGusts" \
			"WindChill" "Humidex" "Tendency" "CurrentWeather" "Image" \
			"TendencyCode" "MslPressure" "CloudAmount" "ObsTime" \
			"VisLocation" "MaxTemp" "MinTemp" "MeanTemp" "Sunrise" "Sunset" \
			"Moonrise" "Moonset" "Precip" "Icon" ]

# Make a request for a city

proc MakeQuery { name } {

	global Data

	set query "$Data(BaseURL)/$Data(Script)"

	switch -exact -- [string index $name 0] {

"K"		{
		set query "$query$Data(Client,USA)$name&cmap=usa"
		} 
"C"		{
		set name [string range $name 1 end]
		set query "$query$Data(Client,Canada)$name"
		}
default	{	
		set query "$query$Data(Client,Canada)$name"
		}
		} ;# End of switch cases

	return "$query$Data(DefaultUnits)"
	}

# Get the station data from the web site

proc LoadStationData { name } {

	global Data tcl_platform

	if { $tcl_platform(platform) == "unix" } {
		set Data(FileName) /tmp/$name[clock seconds].txt
	} else {
		set Data(FileName) $name.txt
		}
		
	if { [catch { ::http::copy [MakeQuery $name] $Data(FileName) } reason] } {
		puts stderr $reason
		return 0
		}
		
	return 1
	}

# Get the observation data

proc GetObservedData { name } {

	set fd [open $name r]
	if { $fd == "" } {
		puts stderr "Failed to open file $name!"
		return ""
		}

	set txt [read $fd]

	close $fd
	
	return [CleanHTMLBlock $txt]
	}

# Strip the HTML tags from a block of data

proc CleanHTMLBlock { txt } {

	regsub -all {<IMG+|<img+} $txt {IMG} txt
	regsub -all {<([^>])+>} $txt {} txt
	regsub -all {^([[:space:]])*\n} $txt {} txt
	regsub -all {  } $txt {} txt
	regsub -all {\n^} $txt {} txt
	
	return $txt
	}

# Get the numeric values from a string

set Data(ExtractNumbers) {([-0123456789.%])+}

proc GetNumberValue { str { width 4 } } {

	global Data

	if { ! [regexp -all $Data(ExtractNumbers) $str value] } {
		set value $Data(MissingValue)
		}

	set len [string length $value]

	while { $len < $width } {
		set value " $value"
		incr len
		}
		
	return $value
	}
	
# Make a time code from the observation time

proc MakeTimeCode { str } {

	set list [split [string trim $str]]
	
	set n "[lindex $list 0] [lindex $list 1] [lindex $list 4] [lindex $list 5] \
				[lindex $list 2]"
				
	regsub -all {[.,]+} $n {} n
	
	return [clock scan $n]
	}

# Parse the wind direction and speed

proc ParseWind { station str hour } {

	global Data

	set parts [split $str " "]
	
	set Data($station,WindDirection,$hour) [FindDirection [lindex $parts 0]]
	if { [string first gust $str] != -1 } {
			regexp -all $Data(ExtractNumbers) $str Data($station,WindSpeed,$hour) Data($station,WindGusts,$hour)
	} else {
			regexp -all $Data(ExtractNumbers) $str Data($station,WindSpeed,$hour)
		}
	}

# Pad out a value

proc Pad { value { len 3 } { with 0 } } {

	set length [string length $value]
	while { $length < $len } {
		set value $with$value
		incr length
		}

	return $value 
	}

# Convert a pressure value to millibars

proc Millibars { value } {

	global Data

	set value [string trimleft $value 0]

	if { $value != $Data(MissingValue) } {
		set value [expr int($value * 100)]
		return [expr $value % 1000]
	} else {
		return $value
		}
	}

# Parse the observation data

proc ParseData { station txt } {

	global Data
	
	set Data(ObsTime) $Data(MissingValue)

	set idx 0
	foreach line [set list [split $txt \n]] {
		if { [string length $line] != 0 } {
			set line [string trim $line]
			set parts [split $line :]
			switch -exact -- [string trim [lindex $parts 0]] {
		
"Observed on"		{	set Data(ValidTime) [lindex $parts 1][lindex $parts 2]
							set Data(ObsTime) [MakeTimeCode $Data(ValidTime)]
					set Data($station,Image,$Data(ObsTime)) ""
					set Data($station,ObsTime,$Data(ObsTime)) $Data(ObsTime)
							}
"Temp."				{	regsub {&deg;+} [lindex $parts 1] {} tmp
							set tmp "$tmp [lindex $parts 2]"
							set Data([MakeKey $station Temperature $Data(ObsTime)]) [GetNumberValue $tmp]
							}

"Pressure"			{	set tmp [FudgePressure [GetNumberValue [lindex $parts 1]]]
						set Data([MakeKey $station MslPressure $Data(ObsTime)]) $tmp
						set Data([MakeKey $station Pressure $Data(ObsTime)]) [Pad [Millibars $tmp]]
							}
"Visibility"		{	set Data([MakeKey $station Visibility $Data(ObsTime)]) [GetNumberValue [lindex $parts 1]]
							set Data([MakeKey $station VisLocation $Data(ObsTime)]) ""
							}
"DewPoint"			{	regsub {&deg;+} [lindex $parts 1] {} tmp
							set tmp "$tmp [lindex $parts 2]"
							set Data([MakeKey $station DewPoint $Data(ObsTime)]) [GetNumberValue $tmp]
							}
"Humidity"			{	set Data([MakeKey $station Humidity $Data(ObsTime)]) [GetNumberValue [lindex $parts 1]]
							}
"Wind"				{	ParseWind $station [lindex $parts 1] $Data(ObsTime)
							}
"WindChill"			{	set Data([MakeKey $station WindChill $Data(ObsTime)]) [GetNumberValue [lindex $parts 1]]
							}
"Humidex"			{	set Data([MakeKey $station Humidex $Data(ObsTime)]) [GetNumberValue [lindex $parts 1]]
							}
"Max Temp."			{	regsub {&deg;+} [lindex $list [expr $idx + 1]] {} Data([MakeKey $station MaxTemp $Data(ObsTime)])
							}
"Min Temp."			{	regsub {&deg;+} [lindex $list [expr $idx + 1]] {} Data([MakeKey $station MinTemp $Data(ObsTime)])
							}
"Mean Temp."		{	regsub {&deg;+} [lindex $list [expr $idx + 1]] {} Data([MakeKey $station MeanTemp $Data(ObsTime)])
							}
"Moonrise"			{	set Data([MakeKey $station Moonrise $Data(ObsTime)]) [lindex $list [expr $idx + 1]]
							}
"Moonset"			{	set Data([MakeKey $station Moonset $Data(ObsTime)]) [lindex $list [expr $idx + 1]]
							}
"Sunrise"			{	set Data([MakeKey $station Sunrise $Data(ObsTime)]) [lindex $list [expr $idx + 1]]
							}
"Sunset"				{	set Data([MakeKey $station Sunset $Data(ObsTime)]) [lindex $list [expr $idx + 1]]
							}
"Precip. Total"	{	set Data([MakeKey $station Precip $Data(ObsTime)]) [lindex $list [expr $idx + 1]]
							}
default				{	if { [string length [string trim $line]] } {  } 
							}
						} ;#End of switch cases
				}
			incr idx	
			}

# If we did not get an observation time, then we probably can't get the data
# for some reason, probably because the station is not valid!
			
	if { $Data(ObsTime) == $Data(MissingValue) } {
		return 0
		}

	set Data($station,Latest) $Data(ObsTime)
	
	set Data([MakeKey $station Tendency $Data(ObsTime)]) $Data(MissingValue)

	ComputePressureTendency $station $Data(ObsTime)
	ComposeCurrentWeather $station $txt $Data(ObsTime)
	
	set Data([MakeKey $station CloudAmount $Data(ObsTime)]) [FindSkyCondition \
				$station $Data([MakeKey $station CurrentWeather $Data(ObsTime)]) $Data(ObsTime)]

	return 1
	}

# Compose Current Weather

proc ComposeCurrentWeather { station txt hour } {

	global Data

	set list [split $txt \n]
	set idx [lsearch -glob $list "*Observed on*"]

	if { $idx != -1 } {

# Get the icon name from the input data

			set icon [lindex [split [lindex $list [incr idx]]  ] 1]
			set icon [string range $icon 8 end]
			regsub -all {\"+} $icon {} Data([MakeKey $station Icon $hour])

# Get the current weather description from the input data

			set Data([MakeKey $station CurrentWeather $hour]) [string trim [lindex $list [incr idx 1]]]

			Log "Icon $Data($station,Icon,$hour) -> $Data($station,CurrentWeather,$hour)"
			
# See if we already have an image for this type of weather

			if { [DownloadIconImage $Data($station,Icon,$hour) $Data($station,CurrentWeather,$hour)] == 0 } {
				puts "No icon for $Data($station,CurrentWeather,$hour) at $station at $hour!"
				set Data($station,Image,$hour) $Data(Images)/29.gif
			} else {
				set Data($station,Image,$hour) $Data(Images)/$Data($station,Icon,$hour)
				}

# Make space for the current weather symbols

			if { [GetData $station CurrentWeather $hour] != "" } {
				set len [expr [string length [GetData $station CurrentWeather $hour]] * 3]
				set Data([MakeKey $station WxLocation $hour]) -$len
				}
				
# Add in the visibility

			if { [GetData $station Visibility $hour] != $Data(MissingValue) } {
				set wx [FindObstructions $station [string tolower [GetData $station CurrentWeather $hour]] $hour]
				if { $wx != "" } {
					set key [MakeKey $station Visibility $hour]
					set Data($key) "$Data($key)$wx"
					set Data([MakeKey $station VisLocation $hour]) "sp -[expr (4 * [string length $wx])] +0"
					}
				}
			}	
	}
	
# Get the tendency code from the change in pressure

proc GetTendencyCode { change } {

	if { $change < 0 } { 
		return 1 
	} else {
		if { $change > 0 } {
			return 3
		} else {
			return 2
			}
		}
	}

# Fake the pressure change

proc FudgePressure { p } {

	return [expr $p + rand()/10.0]
	}

# Compute the pressure change between 2 times

proc PressureChange { station t1 t2 } {

	global Data

	set p1 [GetData $station MslPressure $t1]
	set p2 [GetData $station MslPressure $t2]

	if { $p1 == $Data(MissingValue) || $p2 == $Data(MissingValue) } {
		puts "Missing value for $station p1 = $p1 p2 = $p2"
		return 0
		}

	return [expr $p1 - $p2]
}
	
# Compute the pressure tendency

proc ComputePressureTendency { station hour } {

	global Data
	
	if { [HasDataForField $station Tendency $hour] } { return }

	set previous [expr $hour - (3 * 3600)]
	
	if { [HasDataForField $station MslPressure $hour] } {
		if { [HasDataForField $station MslPressure $previous] } {
			set change [PressureChange $station $hour $previous]
			set Data([MakeKey $station Tendency $hour]) [expr int($change * 100)]
		} else {
			puts "No history data for $station at [clock format $previous -gmt 1]"
			}

		set tm1 [expr $hour - 3600]
		set tm2 [expr $tm1 - 3600]

		if [HasDataForField $station MslPressure $tm2] {
			if [HasDataForField $station MslPressure $tm1] { 
				set change [PressureChange $station $tm1 $tm2]
				set key [MakeKey $station TendencyCode $hour]
				set Data($key) [GetTendencyCode $change]
				set change [PressureChange $station $hour $tm1]
				set Data($key) [expr ($Data($key) << 2) | [GetTendencyCode $change]]
				}
			} 
	} else {
		puts "Data for $station at [clock format $hour -gmt 1] is missing"
		}
	}

# Move a file

proc MoveFile { from to } {

	if { [catch { file copy $from $to } reason] } {
		puts "Failed to move file $from to $to because : $reason"
		}
	}
	
# Download an icon image. The round robin nature of how the CWS does
# its web serving is the reason for this rather bizzare code. You do
# not know which server will deal you the icon, and they use a mix
# of IIS and Apache with different file trees, and different icon sets!

proc DownloadIconImage { icon name } {

	global Data Maps tcl_platform
	
	regsub -all { } $name {_} filename

	set filename $filename.gif

	set result 0

	if { $tcl_platform(platform) == "unix" } {
		set tmpfile tmp[clock seconds].gif
	} else {
		set tmpfile tmp.gif
		}
	
	if { ! [file exists $Data(Images)/$filename] } {
#		puts "Query $Data(BaseURL)/$icon"
		if { [catch { ::http::copy $Data(BaseURL)/$icon $tmpfile } reason] } {
			puts "Failed to get $icon because : $reason"
		} else {
			if { $Maps(IsImage) } {
				MoveFile $tmpfile $Data(Images)/$filename
				if { ! [file exists $Data(Images)/$icon] } {
					MoveFile $tmpfile $Data(Images)/$icon
					}
#				puts "Loaded $icon to $Data(Images)/$filename"
				set result 1
			} else {
				puts "$icon not found on server!"
				}
			}
		catch { file delete $tmpfile } reason	
	} else {
		set result 1
		}

	return $result	
	}
	
# Write a log file

proc Log { msg } {

	global Data
	
	set fd [open $Data(LogFile) a+]
	
	if { $fd != "" } {
		puts $fd "[clock format [clock seconds] ] : $msg"
		close $fd
		}
	}
	
# Log missing fields

proc LogMissingFields { station hour } {

	global Data
	
	set str ""
	foreach name $Data(Names) {
		if { [GetData $station $name $hour] == $Data(MissingValue) } {
			append str "$name "
			}
		}
		
	Log "Missing Items for $station $str"
	}

# Make a key for the database

proc MakeKey { station name hour } {

	return "$station,$name,$hour"
	}

# Check if some valid data is in the database

proc HasDataForField { station name hour } {

	global Data

	set key [MakeKey $station $name $hour]

	if [info exists Data($key)] {
		if { $Data($key) != $Data(MissingValue) } {
			return 1
			}
		}

	return 0
	}

# Save a data element to the database

proc SaveData { station name hour value } {

	global Data

	set key [MakeKey $station $name $hour]
	set Data($key) $value

	}

# Get some data from the database

proc GetData { station name hour } {

	global Data

	set key [MakeKey $station $name $hour]

	if [info exists Data($key)] {
		return $Data($key)
	} else {
		return $Data(MissingValue)
		}
	}
		
# Save a history of data for the station

proc StoreDataHistory { station hour } {

	global Data

	set record ""
	foreach name $Data(Names) {
		if { [GetData $station $name $hour] != $Data(MissingValue) } {
			if { [string length $record] != 0 } {
				append record "*$name=[GetData $station $name $hour]"
			} else {
				set record "$name=[GetData $station $name $hour]"
				}
			}
		}
	
	if { $Data(HasGdbm) } {	
		set db [gdbm open $Data(Database) rwc]
	
		gdbm store $db "$station*$hour" "$record"

		gdbm close $db
	} else {
		}
	
	}

# Update the list of stations

proc UpdateStationList { stationlist } {

	global Data

# Load the data for each of the specified stations

	foreach station [split $stationlist ,] {
		if { [catch { UpdateStation $station } reason] } {
			puts "Failed to update station $station : $reason"
			}
		}
	}
	
# Update the station data

proc UpdateStation { station } {

	global Data
	
	LoadFromDatabase $station
					
# Get the latest data from the station	

	if { [LoadStationData $station] } {
		if { [ParseData $station [GetObservedData $Data(FileName)]] == 0 } {
			Log "Data not available for $station"
			catch { file delete $Data(FileName) }
			return 1
			}
		StoreDataHistory $station $Data(ObsTime)
		catch { file delete $Data(FileName) }
	} else {
		Log "Failed to get data for $station!"
		set Data($station,Model) $Model(MissingData)
		set Data(Update) 1
		set Data($station,Update) 0
		catch { file delete $Data(FileName) }
		return 1
		}

	catch { file delete $Data(FileName) }
	
# Keep a log of the missing fields

	LogMissingFields $station $Data(ObsTime)
	
	}

# Names of the wind directions and their wind directions in degrees...

set Data(Directions) {
	 N 0 NNE 22.5  NE 45 ENE 67.5 E 90 ESE 112.5
	SE 135 SSE 157.5 S 180 SSW 202.5 SW 225
	WSW 247.5 W 270 WNW 292.5 NW 310 NNW 332.5
	}

# Get the wind angle from a name

proc FindDirection { name } {

	global Data

	foreach { n s } $Data(Directions) {
		if { $name == $n } { return $s }
		}

	return 0
	}


# Get the name of an icon

proc GetIconName { name } {

	return [file tail $name]
	}

# Display help information for this script

proc DisplayHelp {} {

	set Help "\nThe format of the command line is:\n\n \
			\tstnplot ?-s? ?stations? ?-m? ?mode? ?-u? ?units?\n \
			where -s or -stations specifies a comma seperated list of stations to watch\n \
			\t\t?-v? ?verbose? ?-w? ?widgets? ?-h? ?history?\n \
			\t\t?-g? ?gmt? ?-d? ?database? ?-??\n\n \
			\t-m or -mode specifies the mode to display the data (plot or depiction)\n \
			\t-u or -units specifies the units to use (imperial or metric)\n \
			\t-v or -verbose specifies whether download information is displayed (yes or no)\n \
			\t-w or -widgets specifies the number of widgets to display (default 4)\n \
			\t-d or -database specifies the database to use for history\n \
			\t-g or -gmt specifies whether time is local or GMT\n \
			\t-h or -history specifies the number of hours of history to keep\n \
			\t-?	or -help displays this help message.\n"

	puts $Help

	Exit
	}
	
# Parse the parameter list

foreach { option value } $argv {
	switch -exact -- [string tolower $option] {

"-d"			-
"-database"	{	set Data(Database) $value
				}
"-g"			-
"-gmt"		{	set Data(GMT) [IsEqual [string tolower $value] yes 1 0]
				}
"-m"			-
"-mode"		{	set Data(DefaultMode) [IsEqual [string tolower $value] plot 0 1]
				}	
"-s"			-
"-station"	{	set Data(Station) [string toupper $value] }

"-u"			-
"-units"		{	set Data(DefaultUnits) [IsEqual [string tolower $value] imperial "&Unit=I" ""]
				}
"-v"			-
"-verbose"	{	set Maps(Verbose) [IsEqual [string tolower $value] "yes"]
				}
"-w"			-
"-widgets"	{	set Data(DefaultWidgets) $value
				}
"-h"			-
"-history"	{	set Data(HistoryLimit) $value
				}
"-?"			-
"-help"		{	DisplayHelp
				}
				
default		{	puts "Option $option is not supported!"
				DisplayHelp
				}
			} ;# End of switch cases
		} ;# End of foreach loop
	
# Check if there is any data for a specified hour

proc HasDataForHour { station hour } {

	global Data

	set hasdata 0
	foreach name $Data(Names) {
		if { [GetData $station $name $hour] != $Data(MissingValue) } {
			incr hasdata
			}
		}
		
	return [IsEqual $hasdata 0 0 1]
	}

# Load the history from the database (Unix version only)

proc LoadFromDatabase { station } {

	global Data

	if { [catch { set db [gdbm open $Data(Database) r] } reason] } {
		puts "No history database available!"
		return
		}

	set list [gdbm list $db -filter $station]

	set now [clock seconds]
	
	foreach item $list {
		set parts [split $item *]
		set valid [lindex $parts 1]

		if { [expr $now - $valid] <= [expr $Data(HistoryLimit) * 3600] } {
			set record [gdbm fetch $db $item]
			foreach name $Data(Names) {
				set Data($station,$name,$valid) $Data(MissingValue)
				}
				
			set elements [split $record *]
			foreach element $elements {
				set l [split $element =]
				set Data($station,[lindex $l 0],$valid) [lindex $l 1]
				}
			}

		}

	gdbm close $db
	}
		 	
# Get the data for the stations

UpdateStationList $Data(Station)


