#!/bin/sh
# \
exec fltkwish "$0" -- ${1+"$@"}
#
# --- stnplot.tcl --- Station plotting application
#
# Copyright(C) I.B.Findleton, 2002. All Rights Reserved.
#
# This script monitors the observed weather data available on the web site
# of the Canadian Weather Service's main public site. The data for the
# selected list of stations is plotted as a standard station plot or is
# displayed as a weather depiction image. The data for the station is
# automatically updated by polling of the web site.
#
# The format of the command line call to this script is:
#
#	stnplot.tcl -s stn,stn,... -v verbose -m mode -w widgets -?
#
# Where the stn values are station identifiers and the verbose parmater is
# either yes or no. The mode value can be plot or depiction. Any number of 
# station identifiers can be provided. The script will wrap the station 
# plots into a scrolled window if there are more than 4 stations specified.
#
# The mode specifies the default display mode. If it is plot, then something
# like the standard station plot model is used. If it is not plot, then an
# image depicting current conditions will be used.
#
# The widgets value is the number of widgets to show in the display window
# without having to scroll the list. Any numbe of stations can be displayed,
# but only this number will be visible at any one time. This is used to 
# conserve window real estate on your desktop.  
#
# Notes:
#	1) Canada is a metric country, so all data is metric unless you
#	   specify the -units imperial option
#
package require TclUtils
package require httpcopy
package require Tclgdbm

set Maps(Verbose) 0

#catch { unset Data }

# Set some default values

set Data(Station) YUL,YMX,YHU,YOW,YSC,YYZ,YQB,KBTV
#set Data(Station) YUL
set Data(BaseURL) www.weatheroffice.ec.gc.ca
set Data(Script) scripts/citygen.pl
set Data(Client,Canada) ?client=ECCDN_e&city=
set Data(Client,USA) ?client=ECUSA_e&city=
set Data(Token) {}
set Data(LogFile) station.log
set Data(HistoryFile) station.dat
set Data(Images) /source/fltk/images/weather
set Data(DefaultMode) 1
set Data(DefaultUnits) ""
set Data(DefaultWidgets) 4
set Data(Database) /source/fltk/observations.gdbm

# Translate the current sky cover to a name for a background image

set SkyConditions {

	{	"Mostly Cloudy"	"Cloudy"					6	}
	{	"Sunny"			"Sunny"					0	}
	{	"Clear"			"Clear"					0	}
	{	"Overcast"		"Cloudy"					8	}
	{  	"Mainly Clear"		"Cloudy_with_clear_periods"	2	}
	{  	"Mainly Sunny"		"Sunny_with_cloudy_periods"	2	}
	{	"Cloudy"			"Cloudy" 					4	}

	}

# Find sky condition

proc FindSkyCondition { str } {

	global SkyConditions Data

	foreach entry $SkyConditions {
		if { [string first "[lindex $entry 0]" $str] != -1 } {
			set Data(Image) $Data(Images)/[lindex $entry 1].bmp
			return [lindex $entry 2]
			}
		}

	return "-1"
	}

# Translate the obstructions to vision. Order is important in this list!
# It appears that this site uses the simple term, like "rain", to mean "light rain"
# and qualifies more intense events with the "moderate" and "heavy" modifiers.

set Obstructions {

	{	"light rainshower"	"RW-" 	"Showers"	}
	{	"moderate rainshower"	"RW"	"Rainy"	}
	{	"heavy rainshower"	"RW+"	"Rainy"	}
	{	"rainshower"		"RW-"	"Showers"	}
	{	"light snowshower"	"SW-"	"Cloudy_with_snow"	}
	{	"moderate snowshower"	"SW"	"Cloudy_with_snow"	}
	{	"heavy snowshower"	"SW+"	"Snowing"	}
	{	"snowshower"		"SW-"	"Cloudy_with_snow"	}
	{	"snow grains"		"SG"		"Cloudy_with_snow"	}
	{	"moderate freezing rain" "ZR" "Freezing_rain"	}
	{	"heavy freezing rain"	"ZR+" "Freezing_rain"	}
	{	"freezing rain"		"ZR-" "Freezing_rain"	}
	{	"light rain"		"R-"		"Rainy"	}
	{	"moderate rain"		"R"	"Rainy"	}
	{	"heavy rain"		"R+"		"Rainy"	}
	{	"rain"			"R-"		"Rainy"	}
	{	"precipitation"		"R-"	"Rainy"	}	
	{	"light snow"		"S-"		"Snowing"		}
	{	"moderate snow"		"S"	"Snowing"	}
	{	"heavy snow"		"S+"		"Snowing"		}
	{	"drifting snow"		"DS"	"Blowing_snow" }
	{	"blowing snow"		"BS"		"Blowing_snow" }
	{	"snow"			"S-"		"Snowing"	}
	{	"light drizzle"		"L-"	"Drizzle"	}
	{	"moderate drizzle"	"L"		"Drizzle"	}
	{	"freezing drizzle"	"ZL-"	"Freezing_rain"	}
	{	"drizzle"			"L-"		"drizzle"	}
	{	"heavy thunderstorm" 	"TRW+" "Thundershowers"	}
	{	"thunderstorm"		"TRW"	"Thundershowers"	}
	{	"thunder"			"T"		"Thundershowers"	}
	{	"haze"			"H"		"Haze"	}
	{	"smoke"			"K"		"Smoke"	}
	{	"mist"			"M"		"Mist"	}
	{	"fog"				"F"	"Foggy"	}
	{	"dust"			"D"		"Dust" }

	}

# Find all obstructions mentioned in a string

proc FindObstructions { str } {

	global Obstructions Data

	set result ""
	
	set list [split $str ,]

	foreach item $list {
		foreach entry $Obstructions {
			if { [regexp -nocase "[lindex $entry 0]" $item] != 0 } {
				append result [lindex $entry 1]
				if { [lindex $entry 2] != "" } {
					set Data(Image) $Data(Images)/[lindex $entry 2].bmp
					}
				break
				}
			}
		}
	return $result
	}

# This is the list of things that can be shown by the station model

set Data(Names) [list "Temperature" "Pressure" "Visibility" "Humidity" \
			"DewPoint" "WindDirection" "WindSpeed" "WindGusts" \
			"WindChill" "Humidex" "Tendency" "CurrentWeather" "Image" \
			"MslPressure" "ObsTime" "ObsDay" "ObsHour"]

# This variable is used to control the state of the station plot widget

set Data(Update) 1

# Make a request for a city

proc MakeQuery { name } {

	global Data

	if { [string index $name 0] == "K" } {
		return "$Data(BaseURL)/$Data(Script)$Data(Client,USA)$name&cmap=usa$Data(DefaultUnits)"
	} else {	
		return "$Data(BaseURL)/$Data(Script)$Data(Client,Canada)$name$Data(DefaultUnits)"
		}
	}

# Get the station data from the web site

proc LoadStationData { name } {

	global Data

	set Data(FileName) $name[clock seconds].txt
		
	if { [catch { ::http::copy [MakeQuery $name] $Data(FileName) } reason] } {
		puts stderr $reason
		return 0
		}
		
	return 1
	}

# Get the observation data

proc GetObservedData { name } {

	set fd [open $name r]
	if { $fd == "" } {
		puts stderr "Failed to open file $name!"
		return ""
		}

	set txt [read $fd]

	close $fd
	
	return [CleanHTMLBlock $txt]
	}

# Strip the HTML tags from a block of data

proc CleanHTMLBlock { txt } {

	regsub -all {<([^>])+>} $txt {} txt
	regsub -all {^([[:space:]])*\n} $txt {} txt
	regsub -all {  } $txt {} txt
	regsub -all {\n^} $txt {} txt
	
	return $txt
	}

# Get the numeric values from a string

set Data(ExtractNumbers) {([-0123456789.%])+}

proc GetNumberValue { str { width 4 } } {

	global Data

	if { ! [regexp -all $Data(ExtractNumbers) $str value] } {
		set value "???"
		}

	set len [string length $value]

	while { $len < $width } {
		set value " $value"
		incr len
		}
		
	return $value
	}

# Parse the wind direction and speed

proc ParseWind { str } {

	global Data

	set parts [split $str " "]
	
	set Data(WindDirection) [FindDirection [lindex $parts 0]]
	if { [string first gust $str] != -1 } {
			regexp -all $Data(ExtractNumbers) $str Data(WindSpeed) Data(WindGusts)
	} else {
			regexp -all $Data(ExtractNumbers) $str Data(WindSpeed);
		}
	}

# Get the observation time

proc GetObsTime { str } {

	global Data

	set list [split $str " ,"]

	set Data(ObsDay) [lindex $list 2]
	set Data(ObsHour) [string range [lindex $list 6] 0 1]
	
	set result "$Data(ObsDay)/$Data(ObsHour)Z"
	
	set Data(ObsHour) [string trimleft $Data(ObsHour) 0]
	
	return $result
	}

# Pad out a value

proc Pad { value { len 3 } { with 0 } } {

	set length [string length $value]
	while { $length < $len } {
		set value $with$value
		incr length
		}

	return $value 
	}

# Convert a pressure value to millibars

proc Millibars { value } {

	set value [string trimleft $value 0]

	if { $value != "???" } {
		set value [expr int($value * 10)]
		return [expr $value % 1000]
	} else {
		return $value
		}
	}

# Parse the observation data

proc ParseData { station txt } {

	global Data
	
	set Data(Image) ""

	foreach line [split $txt \n] {
		if { [string length $line] != 0 } {
			set parts [split $line :]
			switch -exact -- [string trim [lindex $parts 0]] {
		
"Observed on"		{	set Data(ValidTime) [lindex $parts 1][lindex $parts 2]
					set Data(ObsTime) [GetObsTime $Data(ValidTime)]
							}
"Temp."				{	regsub {&deg;+} [lindex $parts 1] {} Data(Temperature)
						set Data(Temperature) "$Data(Temperature) [lindex $parts 2]"
						set Data(Temperature) [GetNumberValue $Data(Temperature)]
							}

"Pressure"			{	set Data(Pressure) [GetNumberValue [lindex $parts 1]]
						set Data(MslPressure) $Data(Pressure)
							}
"Visibility"			{	set Data(Visibility) [GetNumberValue [lindex $parts 1]]
						set Data(VisLocation) ""
							}
"DewPoint"			{	regsub {&deg;+} [lindex $parts 1] {} Data(DewPoint)
						set Data(DewPoint) "$Data(DewPoint) [lindex $parts 2]"
						set Data(DewPoint) [GetNumberValue $Data(DewPoint)]
							}
"Humidity"			{	set Data(Humidity) [GetNumberValue [lindex $parts 1]]
							}
"Wind"				{	set Data(Wind) [lindex $parts 1]
							ParseWind $Data(Wind)
							}
"WindChill"			{	set Data(WindChill) [GetNumberValue [lindex $parts 1]]
						}
"Humidex"				{	set Data(Humidex) [GetNumberValue [lindex $parts 1]]
						}
default				{	if { [string length [string trim $line]] } {  } 
							}
						} ;#End of switch cases
				}	
			}

# If we did not get an observation time, then we probably can't get the data
# for some reason, probably because the station is not valid!
			
	if { [info exists Data(ObsTime)] } {
		if { $Data(ObsTime) == "???" } {
			return 0
			}
	} else {
		return 0
		}

	ComputePressureTendency $station $Data(ObsHour)
	ComposeCurrentWeather $txt
	
	set Data(CloudAmount) [FindSkyCondition $Data(CurrentWeather)]

	return 1
	}

# Check if data for a field was present

proc FieldPresent { name } {

	global Data

	if { [info exists Data($name)] } {
		if { $Data($name) != "???" } {
			return 1
			}
		}

	return 0
	}
	
# Compute the pressure tendency

proc ComputePressureTendency { station hour } {

	global Data
	
	set previous [expr $hour - 3]
	if { $previous < 0 } {
		incr previous 24
		}
	
	if { [FieldPresent MslPressure] } {
		if { [HasDataForField $station MslPressure $previous] } {
			set change [expr [GetNumberValue $Data(MslPressure)] - \
					[GetNumberValue [GetData $station MslPressure $previous]]]
			set Data(Tendency) [Millibars $change]
		} else {
			puts "No history data for $previous"
			}
	} else {
		puts "Data for $hour is $Data(Pressure)"
		}
	}
	
# Compose Current Weather

proc ComposeCurrentWeather { txt } {

	global Data

	set list [split $txt \n]
	set idx [lsearch -glob $list "*Observed on*"]

	if { $idx != -1 } {
			set Data(CurrentWeather) [string trim [lindex $list [incr idx 2]]]
			if { $Data(CurrentWeather) != "" } {
				set len [expr [string length $Data(CurrentWeather)] * 3]
				set Data(WxLocation) -$len
				}
				
			if { $Data(Visibility) != "???" } {
				set wx [FindObstructions [string tolower $Data(CurrentWeather)]]
				if { $wx != "" } {
					set Data(Visibility) "$Data(Visibility)$wx"
					set Data(VisLocation) "sp -[expr (4 * [string length $wx])] +0"
					}
				}
			}	
	}
	
# Draw the sky cover

proc DrawSkyCover {} {

	global Data
	
# An arc drawn with fill active is a pie section of the current background
# color.

	set result "home bg white fill 1 "
	
	switch -exact -- $Data(CloudAmount) {
	
"0"			{	append result "bg blue arc 5 0 360 "
					}
"2"			{	append result "arc 5 0 90 " 
					}
"4"			{	append result "arc 5 0 180 "
					}
"6"			{	append result "arc 5 0 270 "
					}
"8"			{	append result "arc 5 0 360 "
					}
"-1"			{	append result "pc 40 arc 5 0 360 "
					}
				} ;# End of switch cases

	append result "( hm pc black sp -5 +0 rt 90 fd 10 hm sp +0 +5 fd 10 )"
 				
	return $result
	}
	
# Write a log file

proc Log { msg } {

	global Data
	
	set fd [open $Data(LogFile) a+]
	
	if { $fd != "" } {
		puts $fd "[clock format [clock seconds] ] : $msg"
		close $fd
		}
	}
	
# Log missing fields

proc LogMissingFields { station } {

	global Data
	
	set str ""
	foreach name $Data(Names) {
		if { $Data($name) == "???" } {
			append str "$name "
			}
		}
		
	Log "Missing Items for $station $str"
	}

# Make a key for the database

proc MakeKey { station name hour } {

	return "$station,$name,$hour"
	}

# Check if some valid data is in the database

proc HasDataForField { station name hour } {

	global Data

	set key [MakeKey $station $name $hour]

	if [info exists Data($key)] {
		if { $Data($key) != "???" } {
			return 1
			}
		}

	return 0
	}

# Save a data element to the database

proc SaveData { station name hour value } {

	global Data

	set key [MakeKey $station $name $hour]
	set Data($key) $value

	}

# Get some data from the database

proc GetData { station name hour } {

	global Data

	set key [MakeKey $station $name $hour]

	if [info exists Data($key)] {
		return $Data($key)
	} else {
		return "???"
		}
	}
		
# Save a history of data for the station

proc StoreDataHistory { station } {

	global Data
	
	set hour $Data(ObsHour)

	set db [gdbm open $Data(Database) rwc]

	foreach name $Data(Names) {
		if [info exists Data($name)] {
			SaveData $station $name $hour $Data($name)
			gdbm store $db "$station*$name*$Data(ValidTime)" $Data($name)
			}
		}

	gdbm close $db	
	}

# Save the current data to a file

proc SaveDataToFile { stations } {

	global Data

	set fd [open $Data(HistoryFile) w]
	
	if { $fd != "" } {
		close $fd
		}
	}
	
# Load the current display with data from the history list

proc LoadDataHistory { station hour } {

	global Data
	
	foreach name $Data(Names) {
		set Data($name) [GetData $station $name $hour]
		}
	}

# Update the list of stations

proc UpdateStationList { stationlist } {

	global Data

# Deactivate the widget. No recursive clicking!

	set Data(Update) 0

# Cancel any pending updates

	if { $Data(Token) != "" } {
		after cancel $Data(Token)
		}

# Clear any current data

	foreach name $Data(Names) {
		set Data($name) "???"
		}

	foreach station [split $stationlist ,] {
		set idx [lsearch -glob [Windows list] *$station.plot]
		if { $idx != -1 } {
			if { [catch { UpdateStation [lindex [Windows list] $idx] $station } reason] } {
				puts "Failed to update station $station : $reason"
				}
		} else {
			puts "Can't find window for station $station!"
			}
		}
	
# Update the display for this station every 30 minutes at 15 minutes past the hour

	set now [clock seconds]
	set delay [expr (int(($now / 3600) * 3600 + 900)) - $now]
	
	if { $delay < 0 } { set delay [expr 30 * 60] }
	
#	puts "Delay is [expr $delay/60] minutes"
	
	set Data(Token) [after [expr $delay * 1000] "UpdateStationList $stationlist"]

# All done. Activate the widget for input!

	set Data(Update) 1
	}
	
# Update the station data

proc UpdateStation { w station } {

	global Data Model
					
# Get the latest data from the station	

	if { [LoadStationData $station] } {
		if { [ParseData $station [GetObservedData $Data(FileName)]] == 0 } {
			Log "Data not available for $station"
			set Data($station,Model) $Model(NotAvailable)
			set Data(Update) 1
			return 1
			}
		StoreDataHistory $station
	} else {
		Log "Failed to get data for $station!"
		set Data($station,Model) $Model(MissingData)
		set Data(Update) 1
		return 1
		}

# Create the plotting model based on the received data and 
# display it in the window...

	if [catch { BuildPlotModel $station } reason] {
		puts "Failed to build the model for $station : $reason"
		}
	
# Keep a log of the missing fields

	LogMissingFields $station
	
	set l [GetParameter $w Label]
	
	SetParameter $l Hour $Data(ObsHour)

	if [catch { file delete $Data(FileName) } reason] {
		Log "Can't delete file $Data(FileName) because : $reason"
		}
	}

# This is a macro to place an item

proc Item { name where { opts "" } } {

	return "( sp -35 -25 bd 65 50 al $where $opts tx \"\$Data($name)\" )"
	}			

# These turtle graphics scripts will draw a standard station plot style of 
# depiction of the weather data.

set Model(StationCircle) "cs ht home pc black ft helv ( pc red pd cr 6 )"
set Model(Temperature)	"[Item Temperature top,left]"
set Model(Pressure) 	"[Item Pressure top,right]"
set Model(DewPoint)		"[Item DewPoint bottom,left]"
set Model(Visibility)	"( sp -35 +6 \$Data(VisLocation) tx \"\$Data(Visibility)\" )"
set Model(ObsTime)		"( al bottom,centered tx \$Data(ObsTime) )"
set Model(Humidity)		"[Item Humidity bottom,left { pc darkgreen }]"
set Model(WindDirection)	"( pu pc black rt \$Data(WindDirection) fd 6 pd fd 30"
set Model(WindChill)	"[Item WindChill bottom,right { pc blue }]"
set Model(Humidex)		"[Item Humidex bottom,right { pc red }]"
set Model(CurrentWeather) "( al top,centered tx \"\$Data(CurrentWeather)\" )"
set Model(WindGusts)	"( )"
set Model(Tendency)		"[Item Tendency right]"
set Model(Station) 		"( sp -12 -36 tx \"\$station\" )"
set Model(Image)		"( sp -30 -30 im \"\$Data(Image)\" )"
set Model(MissingData)	"cs home ht pc red ( al centered tx \"Data Missing\" )"
set Model(NotAvailable)	"cs home ht pc red ( al centered tx \"Not Available\" )"

# Names of the wind directions and their wind directions in degrees...

set Data(Directions) {
	 N 0 NNE 22.5  NE 45 ENE 67.5 E 90 ESE 112.5
	SE 135 SSE 157.5 S 180 SSW 202.5 SW 225
	WSW 247.5 W 270 WNW 292.5 NW 310 NNW 332.5
	}

# Get the wind angle from a name

proc FindDirection { name } {

	global Data

	foreach { n s } $Data(Directions) {
		if { $name == $n } { return $s }
		}

	return 0
	}

# Make the wind barbs according to the wind speed

proc MakeSpeed { speed } {

	set fast [expr int($speed / 50)]
	set full [expr int(($speed - ($fast * 50)) / 10)]
	set half [expr int(($speed % 10) / 5)]

# The command sequence for the wind barbs is appended to the  commands
# that draw the wind shaft. The barb placement is based on relative
# motion along the shaft.

	set str "lt 180"
	
	while { $fast } {
		append str " ( lt 105 fd 10 rt 177 fd 10 ) pu fd 4 pd"
		incr fast -1
		}

	while { $full } {
		append str " ( lt 105 fd 10 ) pu fd 4 pd"
		incr full -1
		}

	while { $half } {
		append str " ( lt 105 fd 6 ) pu fd 4 pd"
		incr half -1
		}

# This will pop the machine state back to where it was before the shaft
# was drawn.

	append str " )"

	return $str
	}
	
# Build the plot model based on the data available from the station

proc BuildPlotModel { station } {

	global Data 
	global Model

	if { $Data($station,Mode) } {
		set Data(Model) "cs ht"
		append Data(Model) " $Model(CurrentWeather)"
		append Data(Model) " $Model(Image)"
		append Data(Model) " ( al bottom,right tx \"\$Data(ObsTime) \" )"
		append Data(Model) " ( al bottom,left tx \" \$Data(Temperature)\" )" 
	} else {
	
		set Data(Model) $Model(StationCircle)
	
		foreach name $Data(Names) {
			if { $Data($name) != "???" } {
				switch -exact -- $name {

"WindDirection"	{	append Data(Model) " $Model($name) [MakeSpeed $Data(WindSpeed)]"
							}
"MslPressure"		{
					}
"Image"			{
					}
"WindSpeed"		{
					}
"ObsDay"			-
"ObsHour"			{	
					}

"Humidity"		{	if { $Data(DewPoint) == "???" } {
						append Data(Model) " $Model($name)"
						}
					}
default			{	append Data(Model) " $Model($name)"
							}
					}	;# End of switch cases
				}
			}	;# End of foreach loop
		
			append Data(Model) " $Model(ObsTime) [DrawSkyCover]"
		}	;# Mode switch

	set Data(Pressure) [Pad [Millibars $Data(Pressure)]]

#	puts "[subst $Data(Model)]"

	set Data($station,Model) "[subst $Data(Model)]"
	}

# Handle a button press

proc HandleButtonPress { w button x y } {

	$w set -relief sunken

	if { $button == 1 } {
		set station [GetParameter $w Station]
		puts "Update $station"
		if { [catch { UpdateStation $w $station } reason] } {
			puts "Can't update $station : $reason"
			}
	} else {
		source stnplot.tcl
		}

	}

# Display help information for this script

proc DisplayHelp {} {

	set Help "\nThe format of the command line is:\n\n \
			\tstnplot ?-s? ?stations? ?-m? ?mode? ?-u? ?units\n \
			\t\t?-v? ?verbose? ?-w? ?widgets? ?-??\n\n \
			where -s or -stations specifies a comma seperated list of stations to watch\n \
			\t-m or -mode specifies the mode to display the data (plot or depiction)\n \
			\t-u or -units specifies the units to use (imperial or metric)\n \
			\t-v or -verbose specifies whether download information is displayed (yes or no)\n \
			\t-w or -widgets specifies the number of widgets to display (default 4)\n \
			\t-?	or -help displays this help message.\n"

	puts $Help

	Exit
	}
	
# Parse the parameter list

foreach { option value } $argv {
	switch -exact -- [string tolower $option] {

"-d"			-
"-database"	{	set Data(Database) $value
			}
"-m"			-
"-mode"		{	set Data(DefaultMode) [IsEqual [string tolower $value] plot 0 1]
				}	
"-s"			-
"-station"	{	set Data(Station) [string toupper $value] }

"-u"			-
"-units"		{	set Data(DefaultUnits) [IsEqual [string tolower $value] imperial "&Unit=I" ""]
				}
"-v"			-
"-verbose"	{	set Maps(Verbose) [IsEqual [string tolower $value] "yes"]
				}
"-w"			-
"-widgets"	{	set Data(DefaultWidgets) $value
				}

"-?"			-
"-h"			-
"-help"		{	DisplayHelp
				}
				
default		{	puts "Option $option is not supported!"
				DisplayHelp
				}
			} ;# End of switch cases
		} ;# End of foreach loop

# Get a parameter from the label widget

proc GetParameter { w name } {

	set list [split [$w get -data] *]

	set idx [lsearch -glob $list $name=*]

	if { $idx == -1 } { return "" }

	return [lindex [split [lindex $list $idx] =] 1]
	}

# Set a parameter value in a the label widget

proc SetParameter { w name value } {

	set list [split [$w get -data] *]

	set idx [lsearch -glob $list $name=*]

	if { $idx == -1 } {
		lappend list "$name=$value"
		$w set -data [join $list *]
	} else {
		set list [lreplace $list $idx $idx "$name=$value"]
		$w set -data [join $list *]
		}

	}

# Handle a mode change for a station

proc HandleModeChange { w button } {

	global Data

	$w set -relief sunken

	set station [GetParameter $w Station]

	set Data($station,Mode) [expr ($Data($station,Mode) + 1) % 2]

#	puts "$station mode is $Data($station,Mode)"

	set hour [GetParameter $w Hour]

	foreach name $Data(Names) {
		set Data($name) [GetData $station $name $hour]
		}

	BuildPlotModel $station
	}
	
# Display the next observation for a station

proc NextObservation { w button } {

	global Data

	$w set -relief sunken

	set station [GetParameter $w Station]
	set label [GetParameter $w Label]
	
	set hour [expr [GetParameter $label Hour] + 1]
	
	if { $hour > 23 } { set hour 0 }
	
	set hasdata 0
	foreach name $Data(Names) {
		set Data($name) [GetData $station $name $hour]
		if { $Data($name) != "???" } {
			incr hasdata
			}
		}
		
	if { $hasdata } {
		BuildPlotModel $station
		SetParameter $label $hour
	} else {
		puts "No next observation for $station at $hour"
		}					
	
	}
	
# Display the previous observation for a station

proc PreviousObservation { w button } {

	global Data
		
	$w set -relief sunken
	
	set station [GetParameter $w Station]
	
	set label [GetParameter $w Label]
	
	puts $label
	if { [catch { set hour [expr [GetParameter $label Hour] -1] } reason] } {
		puts "Error : $reason"
		}
	
#	puts "Previous for $station is $hour with label $label"
	
	if { $hour < 0 } { incr hour 24 }
	
	set hasdata 0
	foreach name $Data(Names) {
		set Data($name) [GetData $station $name $hour]
		if { $Data($name) != "???" } {
			incr hasdata
			}
		}
		
	if { $hasdata } {
		BuildPlotModel $station
		SetParameter $label $hour
	} else {
		puts "$station has no data for $hour"
		}					
	}

# Load the history from the database

proc LoadFromDatabase { station } {

	global Data

	set db [gdbm open $Data(Database) r]

	set list [gdbm list $db -filter $station]

	foreach item $list {
		set parts [split $item *]
		set name [lindex $parts 1]
		set valid [lindex $parts 2]

		GetObsTime $valid

		set Data($station,$name,$Data(ObsHour)) [gdbm fetch $db $item]

#		puts "$station,$name,$Data(ObsHour) = $Data($station,$name,$Data(ObsHour))"
		}

	gdbm close $db
	}
		 	
# Create an interface to display the data

# Set up some highlight behaviour for the labels

Bind Label <Enter> { %W set -fg red }
Bind Label <Leave> { %W set -fg black }

# Implement mode changes for the plot widgets

Bind Label <ButtonRelease> { %W set -relief raised }
Bind Drawing <ButtonRelease> { %W set -relief raised }

# Make certain we begin from scratch

catch { Destroy t }

# Set up the Scroll if needed. If there are more stations in the list than
# will fit into the specified window, then wrap the list of widgets into
# a Scroll that has only a vertical scroll bar.

set stations [split $Data(Station) ,]
set count [llength $stations]

if { $count < $Data(DefaultWidgets) } {
	set f0 t
} else {
	set itemsize 120 ;# this is the label size plus the drawing size
	set f0 [Scroll t.scroll -ystep $itemsize -scrollbars vertical \
		-x 0 -y 0 -h [expr $Data(DefaultWidgets) * $itemsize]  -w 114]
	}

# Pack all the widgets into something that can be scrolled...

set f [Package $f0.all -o vertical -x 0 -y 0 -w 100]

# Construct a widget for each station.

foreach s $stations {

	LoadFromDatabase $s

	set Data($s,Mode) $Data(DefaultMode)
	set Data($s,Model) "cs"

# Each station is a label and a drawing widget packed into a mega-widget

	set f1 [Package $f.$s -w 100 -orientation vertical]

	set f2 [Package $f1.buttons -w 100 -orientation horizontal]
	
	Label $f2.up -relief raised -text @2< -w 20 \
		-data "Station=$s*Drawing=$f1.plot*Label=$f2.label" \
		-tooltip "Display the previous observation for $s"
	Bind $f2.up <ButtonPress> { PreviousObservation %W %b }
	
	Label $f2.label -relief raised -text $s -w 60 \
		-data "Station=$s*Drawing=$f1.plot" \
		-tooltip "Click to change mode of $s"
	Bind $f2.label <ButtonPress> { HandleModeChange %W %b }

	Label $f2.down -relief raised -text @2> -w 20 \
		-data "Station=$s*Drawing=$f1.plot*Label=$f2.label" \
		-tooltip "Display the next observation for $s"
	Bind $f2.down <ButtonPress> { NextObservation %W %b }

	Drawing $f1.plot  -relief raised -statevariable Data(Update) \
		-tooltip "Click to update the observation for $s" \
		-data "Station=$s*Label=$f2.label" -variable Data($s,Model)

	Bind $f1.plot <ButtonPress> { HandleButtonPress %W %b %x %y}
	}

# Display the resulting UI. Automatic window size is used to create a Toplevel
# widget that wraps the contents.
	
Show t

Wm title t "Stn V1.00"

# Get the data for the stations

UpdateStationList $Data(Station)


