#!/bin/sh
# \
exec fltkwish "$0" ${1+"$@"}
#
# --- timelog.tcl --- Log time activities
#
# Copyright(C) I.B.Findleton, 2003. All Rights Reserved.
#
package require Tclgdbm

set Data(FileName) $env(LOGNAME).timelog.gdbm
set Data(Start) ""
set Data(End) ""
set Data(Text) ""
set Data(Status) ""
set Data(Filter) *
set Data(Project) ""
set Data(Task) ""

# A labeled listbox. This is a good example of how a compound widget can
# be developed from some simpler widgets. Note that the label is set
# to accept only qualified option names, so that the arguments without
# specific qualifiers will not affect the label widget. The Package will
# automatically resize the label according to the width parameters
# specified for the Listbox widget.

proc LabeledListbox { w args } {

	global Data

	set f [Package $w -orientation vertical -relief sunkenframe]
	eval { Label $f.label -relief raised -qn true } $args
	eval { Listbox $f.list -relief flat -bg white } $args

	return $w
	}

# A labeled input area. This compound widget is a text editor with all of
# the usual key bindings and a clipboard interface.

proc Editor { w args } {

	global Data

	set f [Package $w -orientation vertical -relief sunkenframe]
	set f0 [Package $w.label -orientation horizontal]
	eval { Label $f0.label -relief raised -align left,inside -qn true } $args
	eval { LabeledInput $f0.project -relief raised -variable Data(Project) \
		-tooltip "Enter the name of the project" \
		-text Project -ratio .7  -qn true } $args
	eval { LabeledInput $f0.task -relief raised -variable Data(Task) \
		-tooltip "Enter the name of the task" \
		-text Task -ratio .7 -qn true } $args
	eval { Input $f.input -format multi-line -wrap true -relief flat \
		-tooltip "Enter the activity description, press Enter" } $args

	return $w
	}

# Display a message

proc Msg { what } {

	global Data

	set Data(Status) $what
	}

# Compute the duration of a call

proc Duration { } {

	global Data

	set seconds [expr $Data(End) - $Data(Start)]

	return [FormatTimeValue $seconds]
	}

# Format the time value

proc FormatTimeValue { seconds } {

	set hours [expr $seconds / 3600]
	set minutes [expr ($seconds - ($hours * 3600)) / 60]
	set seconds [expr $seconds - ($hours * 3600) - ($minutes * 60)]

	return [format %02d:%02d:%02d $hours $minutes $seconds]
	}

# Handle a call

proc Call { w } {

	global Data

	if { $Data(Start) == "" } {
		set Data(Start) [clock seconds]
		set Data(Text) ""
		$Data(Update) set -state disabled
		$Data(Delete) set -state disabled
		$Data(Quit) set -state disabled
		$Data(Editor) set -command "Call $w"
		$w set -text "End Period"
		Focus $Data(Editor)		
	} else {
		set Data(End) [clock seconds]
		set duration [Duration]
		$Data(KeyList) add [set key [clock format $Data(Start)]*$Data(Project)*$Data(Task)]
		Msg "$key Activity duration $duration"
		set Data(Text) "$Data(Text)\nEnded [clock format $Data(End)] duration $duration"
		gdbm store $Data(Database) $key $Data(Text)
		set Data(Start) ""
		$Data(Quit) set -state normal
		$w set -text "Start Period"
		Focus $w
		}
	}

# Display the selected record

proc Display { w } {

	global Data

	set Data(Text) [gdbm fetch $Data(Database) $Data(Key)]

	$Data(Update) set -state normal
	$Data(Delete) set -state normal
	$Data(Editor) set -command ""
	}

# Delete a record from the database

proc Delete { w } {

	global Data

	gdbm delete $Data(Database) $Data(Key)

	set Data(Text) ""

	$Data(Delete) set -state disabled
	$Data(Update) set -state disabled
#	$Data(Editor) set -command "Call t.all.actions.call"

	Focus $Data(Button)

	Msg "Deleted record for $Data(Key)"

	LoadKeyList $Data(KeyList)
	}

# Update a record

proc Update { w } {

	global Data

	gdbm store $Data(Database) $Data(Key) [$Data(Editor) get -value]

	Msg "Updated record for $Data(Key)"

	}

# Load the list of keys

proc LoadKeyList { w } {

	global Data

	$w clear

	set keys [gdbm list $Data(Database)]

	if [llength $keys] {
		eval { $w add } [lsort [gdbm list $Data(Database)]]
		}
	}

# Display a report

proc Report { w } {

	global Data env

	set Data(Keys) [lsort [$Data(KeyList) contents]]

	Destroy p

	set f [Package p.all -orientation vertical]


	LabeledInput $f.filter -variable Data(Filter) \
		-command "DoReport $f.list.list" -ratio .5 \
		-label "Record Selection Pattern" \
		-tooltip "Pattern used to select keys for the report"
	LabeledListbox $f.list -w 600 -Listbox.h 200 -Label.text "Call Records"

	set f1 [Package $f.actions -orientation horizontal]

	Button $f1.send -text Send -command "Send $f.list.list" \
		-tooltip "Send this report via e-mail"
	Button $f1.print -text Print -command "Print $f.list.list" \
		-tooltip "Print the report"
	Button $f1.save -text Save -command "Save $f.list.list" \
		-tooltip "Save the report to a file"
	
	DoReport $f.list.list

	Show p

	Wm title p "Time Sheet Entry List - $env(LOGNAME)"
	}

# Get the call duration from the entry

proc GetCallDuration { entry } {

	set str [lindex [split $entry] end]

# This crap is related to the Tcl implementation of octal values...groan

	regsub -all 00 $str 0 str
	regsub -all 08 $str 8 str
	regsub -all 09 $str 9 str

	set str [split $str :]
 
	return [expr [lindex $str 0] * 3600 + [lindex $str 1] * 60 + [lindex $str 2]]
	}

# Save the report to a file

proc Save { w { name "" } } {

	global Data env

	if { $name == "" } {
		set name [GetFileName "Save Time Report" *.txt "$env(LOGNAME).pager.txt"]
		}

	if { $name == "" } { 
		Msg "User cancelled save file request"
		return ""
	} else {
		if { [catch { open $name w } fd] } {
			Msg "Failed to open file $name for output"
			return ""
			}
		}

	set list [BuildKeyList [lsort [$Data(KeyList) contents]]]
	
	puts $fd "Time sheet report for $env(LOGNAME) generated [clock format [clock seconds]]\n"

	set total 0

	foreach item $list {
		set item [string trim $item]
		set entry [gdbm fetch $Data(Database) $item]
		incr total [GetCallDuration $entry]
		puts $fd "$item : \n$entry\n"
		}

	puts $fd "Total duration [FormatTimeValue $total]\n" 

	close $fd

	Msg "Report saved to file $name"

	return $name
	}

# Print the report

proc Print { w } {

	global Data env

	set name [Save $w "/tmp/$env(LOGNAME).pager.txt"]

	if { $name == "" } {
		Msg "Print command failed!"
		return ""
		}

	exec cat $name | unix2dos | smbclient //gormenghast/hp m10950 -c "print -"
	}

# Send the report as an e-mail

proc Send { w } {

	global Data env

	set name [Save $w "/tmp/$env(LOGNAME).time.mail"]

	if { $name == "" } {
		Msg "Send command failed!"
		return ""
		}

	set user [GetInput "Enter the e-mail address of the destination :" $env(LOGNAME)@localhost]

	if { [catch { exec mail -s "Page log report" $user < $name } reason] } {
		Msg "Send failed because : $reason"
	} else {
		Msg "Pager report sent to $user"
		}
	}
	
# Get the list of matching keys

proc BuildKeyList { list } {

	global Data

	if { $Data(Filter) == "*" } { return $list }

	set len [llength $list]
	set idx 0
	set result {}

	while { $idx < $len } {
		if { [set jdx [lsearch [lrange $list $idx end] $Data(Filter)]] != -1 } {
			lappend result [lindex $list $jdx]
			set idx [incr jdx]
		} else { break }
		}

	return $result
	}
	

# Build the report

proc DoReport { w } {

	global Data

	set list [BuildKeyList $Data(Keys)]

	$w clear

	foreach item $list {
		if { [catch { gdbm fetch $Data(Database) [string trim $item] } result] } {
			Msg "Record not found for key $item"
		} else {
			$w add "[string trim $item] :"
			eval { $w add } [split $result \n]
			$w add " "
			}
		}
	}
		
		
# Create the GUI

Destroy t p

set f [Package t.all -orientation vertical]

set f1 [Package $f.data -orientation horizontal]

LabeledListbox $f1.keys -variable Data(Key) -height 100 \
	-label.text Records -command "Display %W" -width 205 \
	-tooltip "Select a record to display its contents"
Editor $f1.input -w 400 -h 100 -variable Data(Text) \
	 -label.text "Activity for" 

set Data(KeyList) $f1.keys.list
set Data(Editor) $f1.input.input

Output $f.status -variable Data(Status)

set f5 [Package $f.actions -orientation horizontal]

set Data(Button) [ReturnButton $f5.call -text "Start Period" -command "Call %W" \
		-tooltip "Starts or ends a record entry for an activity"]
set Data(Delete) [Button $f5.delete -text Delete -command "Delete %W" \
		-state disabled -tooltip "Delete the selected record"]
set Data(Update) [Button $f5.update -text Update -command "Update %W" \
		 -state disabled -tooltip "Update the selected record"]
set Data(Report) [Button $f5.report -text Report  \
		-command "Report $Data(KeyList)" \
		-tooltip "Display a selection of records"]
set Data(Quit) [Button $f5.quit -text Exit -command Exit \
		-tooltip "Terminate this program"]

Bind Button <Enter> { %W set -fg orange }
Bind Button <Leave> { %W set -fg black }

Bind t <Destroy> { gdbm close $Data(Database) }

Show t

Wm title t "Time Log V1.00 - $env(LOGNAME)"

# Open the database for this user

if { [catch {
	if { [file exists $Data(FileName)] } {
		set Data(Database) [gdbm open $Data(FileName) rw]
	} else {
		set Data(Database) [gdbm open $Data(FileName) rwc]
		}
		} reason ] } {
	puts stderr "Failed to open $Data(FileName) because : $reason"
	Exit
	}

LoadKeyList $f1.keys.list

Focus $Data(Button)
